# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .context_info import ContextInfo
from .entity_id_base import EntityIdBase
from .event_user_info_base import EventUserInfoBase
from .feedback_info import FeedbackInfo
from .page_info import PageInfo
from .scoped_entity import ScopedEntity
from .session_info import SessionInfo
from .source_info import SourceInfo
from .system_event_name import SystemEventName
from .user_event_name import UserEventName


class EventRequest_UserEvent(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["userEvent"], FieldMetadata(alias="eventType")] = "userEvent"
    id: EntityIdBase
    event_name: typing_extensions.Annotated[UserEventName, FieldMetadata(alias="eventName")]
    user_info: typing_extensions.Annotated[EventUserInfoBase, FieldMetadata(alias="userInfo")]
    feedback_info: typing_extensions.Annotated[
        typing.Optional[typing.List[FeedbackInfo]], FieldMetadata(alias="feedbackInfo")
    ] = None
    page_info: typing_extensions.Annotated[typing.Optional[PageInfo], FieldMetadata(alias="pageInfo")] = None
    timestamp: typing.Optional[dt.datetime] = None
    references: typing.Optional[typing.List[ScopedEntity]] = None
    source_info: typing_extensions.Annotated[typing.Optional[SourceInfo], FieldMetadata(alias="sourceInfo")] = None
    session_info: typing_extensions.Annotated[typing.Optional[SessionInfo], FieldMetadata(alias="sessionInfo")] = None
    context_info: typing_extensions.Annotated[typing.Optional[ContextInfo], FieldMetadata(alias="contextInfo")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EventRequest_SystemEvent(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["systemEvent"], FieldMetadata(alias="eventType")] = (
        "systemEvent"
    )
    id: EntityIdBase
    event_name: typing_extensions.Annotated[SystemEventName, FieldMetadata(alias="eventName")]
    timestamp: typing.Optional[dt.datetime] = None
    references: typing.Optional[typing.List[ScopedEntity]] = None
    source_info: typing_extensions.Annotated[typing.Optional[SourceInfo], FieldMetadata(alias="sourceInfo")] = None
    session_info: typing_extensions.Annotated[typing.Optional[SessionInfo], FieldMetadata(alias="sessionInfo")] = None
    context_info: typing_extensions.Annotated[typing.Optional[ContextInfo], FieldMetadata(alias="contextInfo")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


EventRequest = typing.Union[EventRequest_UserEvent, EventRequest_SystemEvent]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id import EntityId
from ...commons.types.llm_inclusion_status import LlmInclusionStatus
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .base_knowledge_document import BaseKnowledgeDocument


class KnowledgeDocumentSearchResponse(BaseKnowledgeDocument):
    """
    Examples
    --------
    import datetime

    from mavenagi.commons import EntityId
    from mavenagi.knowledge import KnowledgeDocumentSearchResponse

    KnowledgeDocumentSearchResponse(
        knowledge_document_id=EntityId(
            reference_id="how-it-works",
            app_id="help-center",
            organization_id="acme",
            agent_id="support",
            type="KNOWLEDGE_DOCUMENT",
        ),
        knowledge_base_version_id=EntityId(
            reference_id="version-1",
            app_id="help-center",
            organization_id="acme",
            agent_id="support",
            type="KNOWLEDGE_BASE_VERSION",
        ),
        title="How it works",
        llm_inclusion_status="WHEN_RELEVANT",
        url="https://help-center.acme.com/how-it-works",
        created_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2024-02-02 00:00:00+00:00",
        ),
    )
    """

    knowledge_document_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="knowledgeDocumentId")] = (
        pydantic.Field()
    )
    """
    ID that uniquely identifies this knowledge document within its knowledge base
    """

    knowledge_base_version_id: typing_extensions.Annotated[
        typing.Optional[EntityId], FieldMetadata(alias="knowledgeBaseVersionId")
    ] = pydantic.Field(default=None)
    """
    ID that uniquely identifies the knowledge base version that contains this document.
    This may be missing on legacy documents.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the document. Will be shown as part of answers. May be missing on legacy documents.
    """

    llm_inclusion_status: typing_extensions.Annotated[LlmInclusionStatus, FieldMetadata(alias="llmInclusionStatus")] = (
        pydantic.Field()
    )
    """
    Whether the document is included in the agent's knowledge.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    The time at which this document was created.
    """

    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")] = pydantic.Field()
    """
    The time at which this document was last modified.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import BaseOrganization, CreateOrganizationRequest, Organization, OrganizationPatchRequest

__all__ = ["BaseOrganization", "CreateOrganizationRequest", "Organization", "OrganizationPatchRequest"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .entity_id_without_agent import EntityIdWithoutAgent


class ConversationSummary(UniversalBaseModel):
    action_ids: typing_extensions.Annotated[typing.List[EntityIdWithoutAgent], FieldMetadata(alias="actionIds")] = (
        pydantic.Field()
    )
    """
    The IDs of the actions that were taken by Maven in the conversation
    """

    incomplete_action_ids: typing_extensions.Annotated[
        typing.List[EntityIdWithoutAgent], FieldMetadata(alias="incompleteActionIds")
    ] = pydantic.Field()
    """
    The IDs of the actions that were taken by Maven but not completed in the conversation. Occurs when the user is shown an action form but does not submit it.
    """

    insert_count: typing_extensions.Annotated[int, FieldMetadata(alias="insertCount")] = pydantic.Field()
    """
    The number of insert events on messages in the conversation.
    """

    thumbs_up_count: typing_extensions.Annotated[int, FieldMetadata(alias="thumbsUpCount")] = pydantic.Field()
    """
    The number of thumbs up events on messages in the conversation.
    """

    thumbs_down_count: typing_extensions.Annotated[int, FieldMetadata(alias="thumbsDownCount")] = pydantic.Field()
    """
    The number of thumbs down events on messages in the conversation.
    """

    handoff_count: typing_extensions.Annotated[int, FieldMetadata(alias="handoffCount")] = pydantic.Field()
    """
    The number of bot response messages that failed and returned the Agent's system fallback message.
    """

    user_message_count: typing_extensions.Annotated[int, FieldMetadata(alias="userMessageCount")] = pydantic.Field()
    """
    The number of messages of type `USER` in the conversation.
    """

    handle_time: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="handleTime")] = pydantic.Field(
        default=None
    )
    """
    The total time in milliseconds that the user spent interacting with the conversation.
    Calculated by taking the timestamp of the last message in the conversation minus the timestamp of the first message.
    """

    human_agent_response_delay: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="humanAgentResponseDelay")
    ] = pydantic.Field(default=None)
    """
    The time in milliseconds that elapsed before a human agent responded to the conversation.
    Calculated by taking the timestamp of the first message of type `HUMAN_AGENT`
    minus the timestamp of the first message in the conversation.
    
    Will not be provided if the conversation does not have a message of type `HUMAN_AGENT`.
    """

    human_agents: typing_extensions.Annotated[typing.List[str], FieldMetadata(alias="humanAgents")] = pydantic.Field()
    """
    The names of all users that have a message of type `HUMAN_AGENT` on the conversation.
    """

    human_agents_with_inserts: typing_extensions.Annotated[
        typing.List[str], FieldMetadata(alias="humanAgentsWithInserts")
    ] = pydantic.Field()
    """
    The names of all users that have an associated insert event on the conversation.
    """

    users: typing.List[str] = pydantic.Field()
    """
    The names of all users that have a message of type `USER` on the conversation.
    """

    user_identifiers: typing_extensions.Annotated[typing.List[str], FieldMetadata(alias="userIdentifiers")] = (
        pydantic.Field()
    )
    """
    The user identifiers (typically email addresses or phone numbers) of all users that have a message of type `USER` on the conversation.
    """

    last_user_message: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="lastUserMessage")] = (
        pydantic.Field(default=None)
    )
    """
    The text of the last user message in the conversation.
    """

    last_bot_message: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="lastBotMessage")] = (
        pydantic.Field(default=None)
    )
    """
    The text of the last bot message in the conversation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

NumericConversationField = typing.Union[
    typing.Literal[
        "ThumbsUpCount",
        "ThumbsDownCount",
        "InsertCount",
        "UserMessageCount",
        "HandleTime",
        "FirstResponseTime",
        "PredictedNPS",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.app_user_identifier import AppUserIdentifier
from ...commons.types.entity_id import EntityId
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class AppUserSummary(UniversalBaseModel):
    user_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="userId")] = pydantic.Field()
    """
    App provided user ID.
    """

    identifiers: typing.List[AppUserIdentifier] = pydantic.Field()
    """
    App provided identifiers for the user.
    """

    visible_data: typing_extensions.Annotated[typing.Dict[str, str], FieldMetadata(alias="visibleData")] = (
        pydantic.Field()
    )
    """
    App provided data masked according to the data's visibility type. `HIDDEN` or `PARTIALLY_VISIBLE` data values will be not be fully returned.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    Agent,
    AgentEnvironment,
    AgentField,
    AgentFilter,
    AgentPrompting,
    AgentsSearchRequest,
    AgentsSearchResponse,
    CreateAgentRequest,
    PiiCategory,
)

__all__ = [
    "Agent",
    "AgentEnvironment",
    "AgentField",
    "AgentFilter",
    "AgentPrompting",
    "AgentsSearchRequest",
    "AgentsSearchResponse",
    "CreateAgentRequest",
    "PiiCategory",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.not_found_error import NotFoundError
from ..commons.errors.server_error import ServerError
from ..commons.types.error_message import ErrorMessage
from ..commons.types.llm_persona import LlmPersona
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .types.agent import Agent
from .types.agent_environment import AgentEnvironment
from .types.agent_field import AgentField
from .types.agent_filter import AgentFilter
from .types.agents_search_response import AgentsSearchResponse
from .types.pii_category import PiiCategory

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAgentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search(
        self,
        *,
        sort: typing.Optional[AgentField] = OMIT,
        filter: typing.Optional[AgentFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentsSearchResponse]:
        """
        Search for agents across all organizations.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        sort : typing.Optional[AgentField]

        filter : typing.Optional[AgentFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentsSearchResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/agents",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=AgentFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsSearchResponse,
                    parse_obj_as(
                        type_=AgentsSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list(
        self, organization_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[Agent]]:
        """
        Lists all agents for an organization

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Agent]]
            List of all agents for the organization
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[Agent],
                    parse_obj_as(
                        type_=typing.List[Agent],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        name: str,
        environment: AgentEnvironment,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Agent]:
        """
        Create a new agent

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        name : str
            The name of the agent.

        environment : AgentEnvironment
            The environment of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Agent]
            The newly created agent
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents/{jsonable_encoder(agent_reference_id)}",
            method="POST",
            json={
                "name": name,
                "environment": environment,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Agent,
                    parse_obj_as(
                        type_=Agent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Agent]:
        """
        Get an agent

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Agent]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents/{jsonable_encoder(agent_reference_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Agent,
                    parse_obj_as(
                        type_=Agent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def patch(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        environment: typing.Optional[AgentEnvironment] = OMIT,
        default_timezone: typing.Optional[str] = OMIT,
        enabled_pii_categories: typing.Optional[typing.Set[PiiCategory]] = OMIT,
        system_fallback_message: typing.Optional[str] = OMIT,
        persona: typing.Optional[LlmPersona] = OMIT,
        additional_prompt_text: typing.Optional[str] = OMIT,
        category_generation_prompt_text: typing.Optional[str] = OMIT,
        content_safety_violation_response_prompt_text: typing.Optional[str] = OMIT,
        reject_questions_without_knowledge: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Agent]:
        """
        Update mutable agent fields
        All fields will overwrite the existing value on the agent only if provided.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        name : typing.Optional[str]
            The name of the agent.

        environment : typing.Optional[AgentEnvironment]
            The environment of the agent.

        default_timezone : typing.Optional[str]
            The agent's default timezone. This is used when a timezone is not set on a conversation.

        enabled_pii_categories : typing.Optional[typing.Set[PiiCategory]]
            The PII categories that are enabled for the agent.

        system_fallback_message : typing.Optional[str]
            The system fallback message.

        persona : typing.Optional[LlmPersona]
            The overall persona of the agent.

        additional_prompt_text : typing.Optional[str]
            Additional text directly appended to the prompt.

        category_generation_prompt_text : typing.Optional[str]
            LLM prompt for category generation.

        content_safety_violation_response_prompt_text : typing.Optional[str]
            LLM prompt for generating a response when the user's question has been detected as unsafe.

        reject_questions_without_knowledge : typing.Optional[bool]
            Return the system fallback message on all questions that have no relevant knowledge bases or actions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Agent]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents/{jsonable_encoder(agent_reference_id)}",
            method="PATCH",
            json={
                "name": name,
                "environment": environment,
                "defaultTimezone": default_timezone,
                "enabledPiiCategories": enabled_pii_categories,
                "systemFallbackMessage": system_fallback_message,
                "persona": persona,
                "additionalPromptText": additional_prompt_text,
                "categoryGenerationPromptText": category_generation_prompt_text,
                "contentSafetyViolationResponsePromptText": content_safety_violation_response_prompt_text,
                "rejectQuestionsWithoutKnowledge": reject_questions_without_knowledge,
            },
            headers={
                "content-type": "application/merge-patch+json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Agent,
                    parse_obj_as(
                        type_=Agent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Delete an agent.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents/{jsonable_encoder(agent_reference_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAgentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search(
        self,
        *,
        sort: typing.Optional[AgentField] = OMIT,
        filter: typing.Optional[AgentFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentsSearchResponse]:
        """
        Search for agents across all organizations.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        sort : typing.Optional[AgentField]

        filter : typing.Optional[AgentFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentsSearchResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/agents",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=AgentFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsSearchResponse,
                    parse_obj_as(
                        type_=AgentsSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list(
        self, organization_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[Agent]]:
        """
        Lists all agents for an organization

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Agent]]
            List of all agents for the organization
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[Agent],
                    parse_obj_as(
                        type_=typing.List[Agent],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        name: str,
        environment: AgentEnvironment,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Agent]:
        """
        Create a new agent

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        name : str
            The name of the agent.

        environment : AgentEnvironment
            The environment of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Agent]
            The newly created agent
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents/{jsonable_encoder(agent_reference_id)}",
            method="POST",
            json={
                "name": name,
                "environment": environment,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Agent,
                    parse_obj_as(
                        type_=Agent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Agent]:
        """
        Get an agent

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Agent]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents/{jsonable_encoder(agent_reference_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Agent,
                    parse_obj_as(
                        type_=Agent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def patch(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        environment: typing.Optional[AgentEnvironment] = OMIT,
        default_timezone: typing.Optional[str] = OMIT,
        enabled_pii_categories: typing.Optional[typing.Set[PiiCategory]] = OMIT,
        system_fallback_message: typing.Optional[str] = OMIT,
        persona: typing.Optional[LlmPersona] = OMIT,
        additional_prompt_text: typing.Optional[str] = OMIT,
        category_generation_prompt_text: typing.Optional[str] = OMIT,
        content_safety_violation_response_prompt_text: typing.Optional[str] = OMIT,
        reject_questions_without_knowledge: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Agent]:
        """
        Update mutable agent fields
        All fields will overwrite the existing value on the agent only if provided.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        name : typing.Optional[str]
            The name of the agent.

        environment : typing.Optional[AgentEnvironment]
            The environment of the agent.

        default_timezone : typing.Optional[str]
            The agent's default timezone. This is used when a timezone is not set on a conversation.

        enabled_pii_categories : typing.Optional[typing.Set[PiiCategory]]
            The PII categories that are enabled for the agent.

        system_fallback_message : typing.Optional[str]
            The system fallback message.

        persona : typing.Optional[LlmPersona]
            The overall persona of the agent.

        additional_prompt_text : typing.Optional[str]
            Additional text directly appended to the prompt.

        category_generation_prompt_text : typing.Optional[str]
            LLM prompt for category generation.

        content_safety_violation_response_prompt_text : typing.Optional[str]
            LLM prompt for generating a response when the user's question has been detected as unsafe.

        reject_questions_without_knowledge : typing.Optional[bool]
            Return the system fallback message on all questions that have no relevant knowledge bases or actions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Agent]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents/{jsonable_encoder(agent_reference_id)}",
            method="PATCH",
            json={
                "name": name,
                "environment": environment,
                "defaultTimezone": default_timezone,
                "enabledPiiCategories": enabled_pii_categories,
                "systemFallbackMessage": system_fallback_message,
                "persona": persona,
                "additionalPromptText": additional_prompt_text,
                "categoryGenerationPromptText": category_generation_prompt_text,
                "contentSafetyViolationResponsePromptText": content_safety_violation_response_prompt_text,
                "rejectQuestionsWithoutKnowledge": reject_questions_without_knowledge,
            },
            headers={
                "content-type": "application/merge-patch+json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Agent,
                    parse_obj_as(
                        type_=Agent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Delete an agent.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_reference_id)}/agents/{jsonable_encoder(agent_reference_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .agent_user_analytics_request import AgentUserAnalyticsRequest
from .agent_user_column_definition import AgentUserColumnDefinition
from .time_interval import TimeInterval


class AgentUserTableRequest(AgentUserAnalyticsRequest):
    """
    Examples
    --------
    from mavenagi.analytics import (
        AgentUserColumnDefinition,
        AgentUserMetric_Count,
        AgentUserTableRequest,
    )
    from mavenagi.users import AgentUserFilter

    AgentUserTableRequest(
        agent_user_filter=AgentUserFilter(
            search="john",
        ),
        column_definitions=[
            AgentUserColumnDefinition(
                header="user_count",
                metric=AgentUserMetric_Count(),
            )
        ],
    )
    """

    time_grouping: typing_extensions.Annotated[typing.Optional[TimeInterval], FieldMetadata(alias="timeGrouping")] = (
        pydantic.Field(default=None)
    )
    """
    Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
     Example: If set to "DAY," data will be aggregated by day.
    """

    column_definitions: typing_extensions.Annotated[
        typing.List[AgentUserColumnDefinition], FieldMetadata(alias="columnDefinitions")
    ] = pydantic.Field()
    """
    Specifies the metrics to be displayed as columns.
    Only the `count` metric is supported for agent user tables, so each table will have a single column definition using `count`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .feedback_analytics_request import FeedbackAnalyticsRequest
from .feedback_column_definition import FeedbackColumnDefinition
from .feedback_group_by import FeedbackGroupBy
from .time_interval import TimeInterval


class FeedbackTableRequest(FeedbackAnalyticsRequest):
    """
    Examples
    --------
    from mavenagi.analytics import (
        FeedbackColumnDefinition,
        FeedbackGroupBy,
        FeedbackMetric_Count,
        FeedbackTableRequest,
    )
    from mavenagi.conversation import FeedbackFilter

    FeedbackTableRequest(
        feedback_filter=FeedbackFilter(
            types=["THUMBS_UP", "INSERT"],
        ),
        field_groupings=[
            FeedbackGroupBy(
                field="CreatedBy",
            )
        ],
        column_definitions=[
            FeedbackColumnDefinition(
                header="feedback_count",
                metric=FeedbackMetric_Count(),
            )
        ],
    )
    """

    time_grouping: typing_extensions.Annotated[typing.Optional[TimeInterval], FieldMetadata(alias="timeGrouping")] = (
        pydantic.Field(default=None)
    )
    """
    Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
     Example: If set to "DAY," data will be aggregated by day.
    """

    field_groupings: typing_extensions.Annotated[
        typing.List[FeedbackGroupBy], FieldMetadata(alias="fieldGroupings")
    ] = pydantic.Field()
    """
    Specifies the fields by which data should be grouped. Each unique combination forms a row. 
    If multiple fields are provided, the result is grouped by their unique value combinations. 
    If empty, all data is aggregated into a single row. 
    Note: The field CreatedAt should not be used here, all the time-based grouping should be done using the timeGrouping field.
    """

    column_definitions: typing_extensions.Annotated[
        typing.List[FeedbackColumnDefinition], FieldMetadata(alias="columnDefinitions")
    ] = pydantic.Field()
    """
    Specifies the metrics to be displayed as columns.
    Column headers act as keys, with computed metric values as their mapped values.
    There needs to be at least one column definition in the table request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id_base import EntityIdBase
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class InitiateAssetUploadResponse(UniversalBaseModel):
    asset_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="assetId")] = pydantic.Field()
    """
    The ID of the asset. Use this ID to commit the asset after upload.
    """

    upload_url: typing_extensions.Annotated[str, FieldMetadata(alias="uploadUrl")] = pydantic.Field()
    """
    Pre-signed URL for file upload. Use this URL to upload the file directly to storage.
    """

    expires_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="expiresAt")] = pydantic.Field()
    """
    The expiration time for the upload URL
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .conversation_information import ConversationInformation
from .document_information import DocumentInformation
from .entity_id import EntityId
from .inbox_item_fix_add_document import InboxItemFixAddDocument
from .inbox_item_fix_deactivate_document import InboxItemFixDeactivateDocument
from .inbox_item_severity import InboxItemSeverity
from .inbox_item_status import InboxItemStatus


class InboxItem_DuplicateDocuments(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mavenagi.commons import EntityId, InboxItem_Custom

    InboxItem_Custom(
        id=EntityId(
            reference_id="todo-item-1",
            app_id="myapp",
            organization_id="acme",
            agent_id="support",
            type="INBOX_ITEM",
        ),
        status="OPEN",
        severity="HIGH",
        created_at=datetime.datetime.fromisoformat(
            "2025-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2025-02-01 00:00:00+00:00",
        ),
        metadata={"key": "value"},
    )
    """

    type: typing.Literal["duplicateDocuments"] = "duplicateDocuments"
    recommended_fixes: typing_extensions.Annotated[
        typing.List[InboxItemFixDeactivateDocument], FieldMetadata(alias="recommendedFixes")
    ]
    other_fixes: typing_extensions.Annotated[
        typing.List[InboxItemFixDeactivateDocument], FieldMetadata(alias="otherFixes")
    ]
    source_document: typing_extensions.Annotated[DocumentInformation, FieldMetadata(alias="sourceDocument")]
    documents: typing.List[DocumentInformation]
    id: EntityId
    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")]
    status: InboxItemStatus
    severity: InboxItemSeverity

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class InboxItem_MissingKnowledge(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mavenagi.commons import EntityId, InboxItem_Custom

    InboxItem_Custom(
        id=EntityId(
            reference_id="todo-item-1",
            app_id="myapp",
            organization_id="acme",
            agent_id="support",
            type="INBOX_ITEM",
        ),
        status="OPEN",
        severity="HIGH",
        created_at=datetime.datetime.fromisoformat(
            "2025-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2025-02-01 00:00:00+00:00",
        ),
        metadata={"key": "value"},
    )
    """

    type: typing.Literal["missingKnowledge"] = "missingKnowledge"
    fix: InboxItemFixAddDocument
    conversations: typing.List[ConversationInformation]
    id: EntityId
    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")]
    status: InboxItemStatus
    severity: InboxItemSeverity

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class InboxItem_Custom(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mavenagi.commons import EntityId, InboxItem_Custom

    InboxItem_Custom(
        id=EntityId(
            reference_id="todo-item-1",
            app_id="myapp",
            organization_id="acme",
            agent_id="support",
            type="INBOX_ITEM",
        ),
        status="OPEN",
        severity="HIGH",
        created_at=datetime.datetime.fromisoformat(
            "2025-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2025-02-01 00:00:00+00:00",
        ),
        metadata={"key": "value"},
    )
    """

    type: typing.Literal["custom"] = "custom"
    metadata: typing.Dict[str, str]
    id: EntityId
    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")]
    status: InboxItemStatus
    severity: InboxItemSeverity

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
import datetime

from mavenagi.commons import EntityId, InboxItem_Custom

InboxItem_Custom(
    id=EntityId(
        reference_id="todo-item-1",
        app_id="myapp",
        organization_id="acme",
        agent_id="support",
        type="INBOX_ITEM",
    ),
    status="OPEN",
    severity="HIGH",
    created_at=datetime.datetime.fromisoformat(
        "2025-01-01 00:00:00+00:00",
    ),
    updated_at=datetime.datetime.fromisoformat(
        "2025-02-01 00:00:00+00:00",
    ),
    metadata={"key": "value"},
)
"""
InboxItem = typing.Union[InboxItem_DuplicateDocuments, InboxItem_MissingKnowledge, InboxItem_Custom]



# This file was auto-generated by Fern from our API Definition.

import contextlib
import typing
from json.decoder import JSONDecodeError

from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.not_found_error import NotFoundError
from ..commons.errors.server_error import ServerError
from ..commons.types.error_message import ErrorMessage
from ..commons.types.event_field import EventField
from ..commons.types.event_filter import EventFilter
from ..commons.types.event_request import EventRequest
from ..commons.types.event_response import EventResponse
from ..commons.types.events_search_response import EventsSearchResponse
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawEventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, *, request: EventRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[EventResponse]:
        """
        Create a new event

        Parameters
        ----------
        request : EventRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EventResponse]
            A response indicating the event was successfully accepted
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/events",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=EventRequest, direction="write"),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EventResponse,
                    parse_obj_as(
                        type_=EventResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def search(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EventsSearchResponse]:
        """
        Search events

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EventsSearchResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/events/search",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=EventFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EventsSearchResponse,
                    parse_obj_as(
                        type_=EventsSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, event_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[EventResponse]:
        """
        Retrieve details of a specific Event item by its ID.

        Parameters
        ----------
        event_id : str
            The ID of the Event to get.

        app_id : str
            The App ID of the Event to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EventResponse]
            Details of the specified Event
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/events/{jsonable_encoder(event_id)}",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EventResponse,
                    parse_obj_as(
                        type_=EventResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextlib.contextmanager
    def export(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Export events to a CSV file.

        This will output a summary of each event that matches the supplied filter. A maximum of 10,000 events can be exported at a time. For most use cases it is recommended to use the search API instead and convert the JSON response to your desired format. The CSV format may change over time and should not be relied upon by code consumers.

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            A CSV containing one event per row
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/events/export",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=EventFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 404:
                        raise NotFoundError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                ErrorMessage,
                                parse_obj_as(
                                    type_=ErrorMessage,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                ErrorMessage,
                                parse_obj_as(
                                    type_=ErrorMessage,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise ServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                ErrorMessage,
                                parse_obj_as(
                                    type_=ErrorMessage,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()


class AsyncRawEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, request: EventRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[EventResponse]:
        """
        Create a new event

        Parameters
        ----------
        request : EventRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EventResponse]
            A response indicating the event was successfully accepted
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/events",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=EventRequest, direction="write"),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EventResponse,
                    parse_obj_as(
                        type_=EventResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def search(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EventsSearchResponse]:
        """
        Search events

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EventsSearchResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/events/search",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=EventFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EventsSearchResponse,
                    parse_obj_as(
                        type_=EventsSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, event_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[EventResponse]:
        """
        Retrieve details of a specific Event item by its ID.

        Parameters
        ----------
        event_id : str
            The ID of the Event to get.

        app_id : str
            The App ID of the Event to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EventResponse]
            Details of the specified Event
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/events/{jsonable_encoder(event_id)}",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EventResponse,
                    parse_obj_as(
                        type_=EventResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextlib.asynccontextmanager
    async def export(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Export events to a CSV file.

        This will output a summary of each event that matches the supplied filter. A maximum of 10,000 events can be exported at a time. For most use cases it is recommended to use the search API instead and convert the JSON response to your desired format. The CSV format may change over time and should not be relied upon by code consumers.

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            A CSV containing one event per row
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/events/export",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=EventFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 404:
                        raise NotFoundError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                ErrorMessage,
                                parse_obj_as(
                                    type_=ErrorMessage,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                ErrorMessage,
                                parse_obj_as(
                                    type_=ErrorMessage,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise ServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                ErrorMessage,
                                parse_obj_as(
                                    type_=ErrorMessage,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class BasePaginatedRequest(UniversalBaseModel):
    page: typing.Optional[int] = pydantic.Field(default=None)
    """
    Page number to return, defaults to 0
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size of the page to return, defaults to 20
    """

    sort_desc: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="sortDesc")] = pydantic.Field(
        default=None
    )
    """
    Whether to sort descending, defaults to true
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .browser_type import BrowserType


class BrowserInfo(UniversalBaseModel):
    type: BrowserType
    name: typing.Optional[str] = None
    version: typing.Optional[str] = None
    user_agent: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="userAgent")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from ...core.serialization import FieldMetadata
from .segment_status import SegmentStatus


class SegmentPatchRequest(UniversalBaseModel):
    app_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="appId")] = pydantic.Field(
        default=None
    )
    """
    The App ID of the segment to update. If not provided, the ID of the calling app will be used.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the segment.
    """

    precondition: typing.Optional["Precondition"] = pydantic.Field(default=None)
    """
    The precondition that must be met for a conversation message to be included in the segment.
    """

    status: typing.Optional[SegmentStatus] = pydantic.Field(default=None)
    """
    The status of the segment. Segments can only be deactivated if they are not set on any actions or active knowledge bases.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from ...commons.types.precondition_group import PreconditionGroup  # noqa: E402, F401, I001
from ...commons.types.precondition import Precondition  # noqa: E402, F401, I001

update_forward_refs(SegmentPatchRequest)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...commons.types.inbox_item_status import InboxItemStatus
from ...commons.types.inbox_item_type import InboxItemType
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class InboxFilter(UniversalBaseModel):
    statuses: typing.Optional[typing.List[InboxItemStatus]] = pydantic.Field(default=None)
    """
    List of inbox item statuses to filter by.
    """

    type: typing.Optional[typing.List[InboxItemType]] = pydantic.Field(default=None)
    """
    List of inbox item types to filter by.
    """

    created_after: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAfter")] = (
        pydantic.Field(default=None)
    )
    """
    Filter for items created after this timestamp.
    """

    created_before: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdBefore")] = (
        pydantic.Field(default=None)
    )
    """
    Filter for items created before this timestamp.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .conversation_analysis import ConversationAnalysis
from .conversation_summary import ConversationSummary
from .entity_id import EntityId
from .response_config import ResponseConfig
from .simulation_context import SimulationContext


class BaseConversationResponse(UniversalBaseModel):
    response_config: typing_extensions.Annotated[
        typing.Optional[ResponseConfig], FieldMetadata(alias="responseConfig")
    ] = pydantic.Field(default=None)
    """
    Optional configurations for responses to this conversation
    """

    subject: typing.Optional[str] = pydantic.Field(default=None)
    """
    The subject of the conversation
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The url of the conversation
    """

    created_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")] = (
        pydantic.Field(default=None)
    )
    """
    The date and time the conversation was created
    """

    updated_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="updatedAt")] = (
        pydantic.Field(default=None)
    )
    """
    The date and time the conversation was last updated
    """

    tags: typing.Optional[typing.Set[str]] = pydantic.Field(default=None)
    """
    The tags of the conversation. Used for filtering in Agent Designer.
    """

    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    The metadata of the conversation supplied by the app which created the conversation.
    """

    all_metadata: typing_extensions.Annotated[
        typing.Dict[str, typing.Dict[str, str]], FieldMetadata(alias="allMetadata")
    ] = pydantic.Field()
    """
    All metadata for the conversation. Keyed by appId.
    """

    conversation_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationId")] = pydantic.Field()
    """
    The ID that uniquely identifies this conversation
    """

    analysis: ConversationAnalysis = pydantic.Field()
    """
    An analysis of the conversation. Fields are generated by Maven via an analysis of user messages. This object is calculated on a delay. Fields will not be up to date on ask requests.
    """

    summary: ConversationSummary = pydantic.Field()
    """
    A summary of the conversation. Fields are calculated from conversation data. Unlike analysis, all fields can be derived from other data available in the API. This object is provided as a convenience and is calculated on a delay. Fields will not be up to date on ask requests.
    """

    deleted: bool = pydantic.Field()
    """
    Whether the conversation user-specific data has been deleted. See `deleteConversation` for details.
    """

    open: bool = pydantic.Field()
    """
    Whether the conversation is able to receive asynchronous messages.
    Only applicable if a conversation is initialized with the `ASYNC` capability. Defaults to true. Can be closed using the `PATCH` API.
    """

    llm_enabled: typing_extensions.Annotated[bool, FieldMetadata(alias="llmEnabled")] = pydantic.Field()
    """
    Whether the LLM is enabled for this conversation.
    If true, `USER` messages sent via the ask API will be sent to the LLM and a `BOT_RESPONSE` or `BOT_SUGGESTION` message will be generated.
    If false, `USER` messages will not be sent to the LLM.
    """

    simulation_context: typing_extensions.Annotated[
        typing.Optional[SimulationContext], FieldMetadata(alias="simulationContext")
    ] = pydantic.Field(default=None)
    """
    Additional context used for simulation runs. When present, this conversation is treated as a simulation. 
    Simulation conversations are excluded from normal search results unless explicitly included via the `simulationFilter` field.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...knowledge.types.knowledge_document_request import KnowledgeDocumentRequest


class AddDocumentFixRequest(UniversalBaseModel):
    knowledge_document_request: typing_extensions.Annotated[
        KnowledgeDocumentRequest, FieldMetadata(alias="knowledgeDocumentRequest")
    ] = pydantic.Field()
    """
    Knowledge document to be added by this request
    """

    knowledge_base_reference_id: typing_extensions.Annotated[str, FieldMetadata(alias="knowledgeBaseReferenceId")] = (
        pydantic.Field()
    )
    """
    Reference id of the Knowledge Base the document will be added to. 
    The appId is inferred from the request. Apps can only add documents to their own knowledge bases.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .knowledge_base_version import KnowledgeBaseVersion


class KnowledgeBaseVersionsListResponse(UniversalBaseModel):
    knowledge_base_versions: typing_extensions.Annotated[
        typing.List[KnowledgeBaseVersion], FieldMetadata(alias="knowledgeBaseVersions")
    ] = pydantic.Field()
    """
    The active knowledge base versions for the knowledge base. Includes all versions within the last 90 days or the last 10, whichever is greater.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from .agent_user_row import AgentUserRow
from .table_response_base import TableResponseBase


class AgentUserTableResponse(TableResponseBase):
    """
    Examples
    --------
    import datetime

    from mavenagi.analytics import (
        AgentUserRow,
        AgentUserTableResponse,
        CellData_Double,
        FieldValue_DateTime,
    )

    AgentUserTableResponse(
        headers=["user_count"],
        rows=[
            AgentUserRow(
                identifier={
                    "CreatedAt": FieldValue_DateTime(
                        value=datetime.datetime.fromisoformat(
                            "2023-10-01 00:00:00+00:00",
                        )
                    )
                },
                data={"user_count": CellData_Double(value=5.0)},
            )
        ],
    )
    """

    rows: typing.List[AgentUserRow] = pydantic.Field()
    """
    Each row contains the user count for the given search query and corresponding time grouping.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .action_enum_option import ActionEnumOption
from .action_o_auth_configuration import ActionOAuthConfiguration
from .action_parameter_type import ActionParameterType


class ActionParameter(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID field will be used when parameters are supplied to `executeAction`.
    """

    label: str = pydantic.Field()
    """
    The title of the field that will be shown on action forms.
    """

    description: str = pydantic.Field()
    """
    A longer description of the field which will be shown in smaller text near the label on action forms.
    """

    required: bool = pydantic.Field()
    """
    Whether the field is required for action execution.
    """

    hidden: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When user interaction is required, whether this parameter should be excluded from forms. Hidden parameters are not displayed to users but their values are still populated by the LLM and sent to actions. Defaults to false.
    """

    type: typing.Optional[ActionParameterType] = pydantic.Field(default=None)
    """
    The parameter type. Values provided to `executeAction` will conform to this type. Defaults to `STRING`.
    """

    enum_options: typing_extensions.Annotated[
        typing.Optional[typing.List[ActionEnumOption]], FieldMetadata(alias="enumOptions")
    ] = pydantic.Field(default=None)
    """
    Restricts the action parameter to only the options in this list. Valid for type `STRING`, `BOOLEAN`, and `NUMBER`. Should not be used when type is `SCHEMA`.
    """

    schema_: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="schema")] = pydantic.Field(
        default=None
    )
    """
    JSON schema for validating the parameter value. Only valid when type is `SCHEMA`.
    
    **Schema Requirements:**
    - Root type must be "object" - The top-level schema must define an object type
    - Supported types: string, number, boolean, integer, object, array, enum, anyOf
    - Unsupported features: oneOf, allOf, const, not, if/then/else, pattern, maxLength, minLength
    - Required fields: All fields must be required (no optional properties)
    - Additional properties: Must be set to false
    
    **Limits:**
    - Maximum nesting depth: 5 levels
    - Maximum total properties: 100
    - Maximum total string length: 15,000 characters
    - Maximum total enum values: 500
    """

    oauth_configuration: typing_extensions.Annotated[
        typing.Optional[ActionOAuthConfiguration], FieldMetadata(alias="oauthConfiguration")
    ] = pydantic.Field(default=None)
    """
    OAuth configuration required to start an OAuth authorization flow when this parameter's type is `OAUTH`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .add_document_fix_request import AddDocumentFixRequest
from .apply_fixes_request import ApplyFixesRequest
from .inbox_filter import InboxFilter
from .inbox_search_request import InboxSearchRequest
from .inbox_search_response import InboxSearchResponse

__all__ = ["AddDocumentFixRequest", "ApplyFixesRequest", "InboxFilter", "InboxSearchRequest", "InboxSearchResponse"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...commons.types.conversation_preview import ConversationPreview
from ...commons.types.page import Page
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ConversationsResponse(Page):
    conversations: typing.List[ConversationPreview] = pydantic.Field()
    """
    The conversations that match the search criteria
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id import EntityId
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .segment_base import SegmentBase
from .segment_status import SegmentStatus


class SegmentResponse(SegmentBase):
    """
    Examples
    --------
    from mavenagi.commons import EntityId, Precondition_Group, Precondition_User
    from mavenagi.segments import SegmentResponse

    SegmentResponse(
        segment_id=EntityId(
            reference_id="admin-users",
            app_id="auth-app",
            organization_id="acme",
            agent_id="support",
            type="SEGMENT",
        ),
        name="Admin users",
        status="ACTIVE",
        precondition=Precondition_Group(
            operator="AND",
            preconditions=[
                Precondition_User(
                    key="userKey",
                ),
                Precondition_User(
                    key="userKey2",
                ),
            ],
        ),
    )
    """

    segment_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="segmentId")] = pydantic.Field()
    """
    ID that uniquely identifies this segment
    """

    status: SegmentStatus = pydantic.Field()
    """
    Whether or not the segment is in active use. To preserve historical data, segments can not be deleted.
    
    Only active segments will be evaluated for matching user questions.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from ...commons.types.precondition_group import PreconditionGroup  # noqa: E402, F401, I001



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .browser_info import BrowserInfo
from .device_info import DeviceInfo
from .geo_info import GeoInfo
from .ip_info import IpInfo
from .language_info import LanguageInfo
from .source_type import SourceType


class SourceInfo(UniversalBaseModel):
    type: SourceType
    device_info: typing_extensions.Annotated[typing.Optional[DeviceInfo], FieldMetadata(alias="deviceInfo")] = None
    browser_info: typing_extensions.Annotated[typing.Optional[BrowserInfo], FieldMetadata(alias="browserInfo")] = None
    geo_info: typing_extensions.Annotated[typing.Optional[GeoInfo], FieldMetadata(alias="geoInfo")] = None
    ip_info: typing_extensions.Annotated[typing.Optional[IpInfo], FieldMetadata(alias="ipInfo")] = None
    language_info: typing_extensions.Annotated[typing.Optional[LanguageInfo], FieldMetadata(alias="languageInfo")] = (
        None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id_without_agent import EntityIdWithoutAgent
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .knowledge_base_version_finalize_status import KnowledgeBaseVersionFinalizeStatus


class FinalizeKnowledgeBaseVersionRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.commons import EntityIdWithoutAgent
    from mavenagi.knowledge import FinalizeKnowledgeBaseVersionRequest

    FinalizeKnowledgeBaseVersionRequest(
        version_id=EntityIdWithoutAgent(
            type="KNOWLEDGE_BASE_VERSION",
            reference_id="versionId",
            app_id="maven",
        ),
        status="SUCCEEDED",
    )
    """

    version_id: typing_extensions.Annotated[typing.Optional[EntityIdWithoutAgent], FieldMetadata(alias="versionId")] = (
        pydantic.Field(default=None)
    )
    """
    ID that uniquely identifies which knowledge base version to finalize. If not provided will use the most recent version of the knowledge base.
    """

    status: typing.Optional[KnowledgeBaseVersionFinalizeStatus] = pydantic.Field(default=None)
    """
    Whether the knowledge base version processing was successful or not.
    """

    error_message: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="errorMessage")] = (
        pydantic.Field(default=None)
    )
    """
    A user-facing error message that provides more details about a version failure.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .event_type import EventType
from .feedback_type import FeedbackType
from .scoped_entity import ScopedEntity
from .system_event_name import SystemEventName
from .user_event_name import UserEventName


class EventFilter(UniversalBaseModel):
    search: typing.Optional[str] = None
    created_after: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAfter")] = None
    created_before: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdBefore")] = (
        None
    )
    references: typing.Optional[typing.List[ScopedEntity]] = None
    event_types: typing_extensions.Annotated[
        typing.Optional[typing.List[EventType]], FieldMetadata(alias="eventTypes")
    ] = None
    user_event_names: typing_extensions.Annotated[
        typing.Optional[typing.List[UserEventName]], FieldMetadata(alias="userEventNames")
    ] = None
    system_event_names: typing_extensions.Annotated[
        typing.Optional[typing.List[SystemEventName]], FieldMetadata(alias="systemEventNames")
    ] = None
    agent_user_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="agentUserIds")
    ] = None
    legacy_feedback_types: typing_extensions.Annotated[
        typing.Optional[typing.List[FeedbackType]], FieldMetadata(alias="legacyFeedbackTypes")
    ] = None
    has_event_text: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="hasEventText")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.attachment_response import AttachmentResponse
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .knowledge_document_search_response import KnowledgeDocumentSearchResponse
from .knowledge_document_status import KnowledgeDocumentStatus


class KnowledgeDocumentResponse(KnowledgeDocumentSearchResponse):
    """
    Examples
    --------
    import datetime

    from mavenagi.commons import EntityId
    from mavenagi.knowledge import KnowledgeDocumentResponse

    KnowledgeDocumentResponse(
        knowledge_document_id=EntityId(
            reference_id="getting-started",
            app_id="readme",
            organization_id="acme",
            agent_id="support",
            type="KNOWLEDGE_DOCUMENT",
        ),
        knowledge_base_version_id=EntityId(
            reference_id="versionId",
            app_id="maven",
            organization_id="acme",
            agent_id="support",
            type="KNOWLEDGE_BASE_VERSION",
        ),
        content="## Getting started This is a getting started guide for the help center.",
        title="Getting started",
        metadata={"category": "getting-started"},
        created_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2024-02-02 00:00:00+00:00",
        ),
        llm_inclusion_status="WHEN_RELEVANT",
    )
    """

    processing_status: typing_extensions.Annotated[
        typing.Optional[KnowledgeDocumentStatus], FieldMetadata(alias="processingStatus")
    ] = pydantic.Field(default=None)
    """
    The current processing status of the knowledge document
    """

    content: str = pydantic.Field()
    """
    The content of the document in markdown format. Not shown directly to users.
    """

    asset: typing.Optional[AttachmentResponse] = pydantic.Field(default=None)
    """
    If the document is associated with an asset, this will contain the asset metadata
    """

    metadata: typing.Dict[str, str] = pydantic.Field()
    """
    Metadata for the knowledge document.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..commons.types.llm_persona import LlmPersona
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawAgentsClient, RawAgentsClient
from .types.agent import Agent
from .types.agent_environment import AgentEnvironment
from .types.agent_field import AgentField
from .types.agent_filter import AgentFilter
from .types.agents_search_response import AgentsSearchResponse
from .types.pii_category import PiiCategory

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AgentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAgentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAgentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAgentsClient
        """
        return self._raw_client

    def search(
        self,
        *,
        sort: typing.Optional[AgentField] = OMIT,
        filter: typing.Optional[AgentFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsSearchResponse:
        """
        Search for agents across all organizations.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        sort : typing.Optional[AgentField]

        filter : typing.Optional[AgentFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsSearchResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agents.search()
        """
        _response = self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    def list(
        self, organization_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Agent]:
        """
        Lists all agents for an organization

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Agent]
            List of all agents for the organization

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agents.list(
            organization_reference_id="organizationReferenceId",
        )
        """
        _response = self._raw_client.list(organization_reference_id, request_options=request_options)
        return _response.data

    def create(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        name: str,
        environment: AgentEnvironment,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Agent:
        """
        Create a new agent

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        name : str
            The name of the agent.

        environment : AgentEnvironment
            The environment of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Agent
            The newly created agent

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agents.create(
            organization_reference_id="organizationReferenceId",
            agent_reference_id="agentReferenceId",
            name="name",
            environment="DEMO",
        )
        """
        _response = self._raw_client.create(
            organization_reference_id,
            agent_reference_id,
            name=name,
            environment=environment,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Agent:
        """
        Get an agent

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Agent

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agents.get(
            organization_reference_id="organizationReferenceId",
            agent_reference_id="agentReferenceId",
        )
        """
        _response = self._raw_client.get(organization_reference_id, agent_reference_id, request_options=request_options)
        return _response.data

    def patch(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        environment: typing.Optional[AgentEnvironment] = OMIT,
        default_timezone: typing.Optional[str] = OMIT,
        enabled_pii_categories: typing.Optional[typing.Set[PiiCategory]] = OMIT,
        system_fallback_message: typing.Optional[str] = OMIT,
        persona: typing.Optional[LlmPersona] = OMIT,
        additional_prompt_text: typing.Optional[str] = OMIT,
        category_generation_prompt_text: typing.Optional[str] = OMIT,
        content_safety_violation_response_prompt_text: typing.Optional[str] = OMIT,
        reject_questions_without_knowledge: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Agent:
        """
        Update mutable agent fields
        All fields will overwrite the existing value on the agent only if provided.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        name : typing.Optional[str]
            The name of the agent.

        environment : typing.Optional[AgentEnvironment]
            The environment of the agent.

        default_timezone : typing.Optional[str]
            The agent's default timezone. This is used when a timezone is not set on a conversation.

        enabled_pii_categories : typing.Optional[typing.Set[PiiCategory]]
            The PII categories that are enabled for the agent.

        system_fallback_message : typing.Optional[str]
            The system fallback message.

        persona : typing.Optional[LlmPersona]
            The overall persona of the agent.

        additional_prompt_text : typing.Optional[str]
            Additional text directly appended to the prompt.

        category_generation_prompt_text : typing.Optional[str]
            LLM prompt for category generation.

        content_safety_violation_response_prompt_text : typing.Optional[str]
            LLM prompt for generating a response when the user's question has been detected as unsafe.

        reject_questions_without_knowledge : typing.Optional[bool]
            Return the system fallback message on all questions that have no relevant knowledge bases or actions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Agent

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agents.patch(
            organization_reference_id="organizationReferenceId",
            agent_reference_id="agentReferenceId",
        )
        """
        _response = self._raw_client.patch(
            organization_reference_id,
            agent_reference_id,
            name=name,
            environment=environment,
            default_timezone=default_timezone,
            enabled_pii_categories=enabled_pii_categories,
            system_fallback_message=system_fallback_message,
            persona=persona,
            additional_prompt_text=additional_prompt_text,
            category_generation_prompt_text=category_generation_prompt_text,
            content_safety_violation_response_prompt_text=content_safety_violation_response_prompt_text,
            reject_questions_without_knowledge=reject_questions_without_knowledge,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete an agent.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agents.delete(
            organization_reference_id="organizationReferenceId",
            agent_reference_id="agentReferenceId",
        )
        """
        _response = self._raw_client.delete(
            organization_reference_id, agent_reference_id, request_options=request_options
        )
        return _response.data


class AsyncAgentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAgentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAgentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAgentsClient
        """
        return self._raw_client

    async def search(
        self,
        *,
        sort: typing.Optional[AgentField] = OMIT,
        filter: typing.Optional[AgentFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsSearchResponse:
        """
        Search for agents across all organizations.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        sort : typing.Optional[AgentField]

        filter : typing.Optional[AgentFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsSearchResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agents.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    async def list(
        self, organization_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Agent]:
        """
        Lists all agents for an organization

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Agent]
            List of all agents for the organization

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agents.list(
                organization_reference_id="organizationReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(organization_reference_id, request_options=request_options)
        return _response.data

    async def create(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        name: str,
        environment: AgentEnvironment,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Agent:
        """
        Create a new agent

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        name : str
            The name of the agent.

        environment : AgentEnvironment
            The environment of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Agent
            The newly created agent

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agents.create(
                organization_reference_id="organizationReferenceId",
                agent_reference_id="agentReferenceId",
                name="name",
                environment="DEMO",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            organization_reference_id,
            agent_reference_id,
            name=name,
            environment=environment,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Agent:
        """
        Get an agent

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Agent

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agents.get(
                organization_reference_id="organizationReferenceId",
                agent_reference_id="agentReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            organization_reference_id, agent_reference_id, request_options=request_options
        )
        return _response.data

    async def patch(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        environment: typing.Optional[AgentEnvironment] = OMIT,
        default_timezone: typing.Optional[str] = OMIT,
        enabled_pii_categories: typing.Optional[typing.Set[PiiCategory]] = OMIT,
        system_fallback_message: typing.Optional[str] = OMIT,
        persona: typing.Optional[LlmPersona] = OMIT,
        additional_prompt_text: typing.Optional[str] = OMIT,
        category_generation_prompt_text: typing.Optional[str] = OMIT,
        content_safety_violation_response_prompt_text: typing.Optional[str] = OMIT,
        reject_questions_without_knowledge: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Agent:
        """
        Update mutable agent fields
        All fields will overwrite the existing value on the agent only if provided.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        name : typing.Optional[str]
            The name of the agent.

        environment : typing.Optional[AgentEnvironment]
            The environment of the agent.

        default_timezone : typing.Optional[str]
            The agent's default timezone. This is used when a timezone is not set on a conversation.

        enabled_pii_categories : typing.Optional[typing.Set[PiiCategory]]
            The PII categories that are enabled for the agent.

        system_fallback_message : typing.Optional[str]
            The system fallback message.

        persona : typing.Optional[LlmPersona]
            The overall persona of the agent.

        additional_prompt_text : typing.Optional[str]
            Additional text directly appended to the prompt.

        category_generation_prompt_text : typing.Optional[str]
            LLM prompt for category generation.

        content_safety_violation_response_prompt_text : typing.Optional[str]
            LLM prompt for generating a response when the user's question has been detected as unsafe.

        reject_questions_without_knowledge : typing.Optional[bool]
            Return the system fallback message on all questions that have no relevant knowledge bases or actions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Agent

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agents.patch(
                organization_reference_id="organizationReferenceId",
                agent_reference_id="agentReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(
            organization_reference_id,
            agent_reference_id,
            name=name,
            environment=environment,
            default_timezone=default_timezone,
            enabled_pii_categories=enabled_pii_categories,
            system_fallback_message=system_fallback_message,
            persona=persona,
            additional_prompt_text=additional_prompt_text,
            category_generation_prompt_text=category_generation_prompt_text,
            content_safety_violation_response_prompt_text=content_safety_violation_response_prompt_text,
            reject_questions_without_knowledge=reject_questions_without_knowledge,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self,
        organization_reference_id: str,
        agent_reference_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete an agent.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The ID of the organization.

        agent_reference_id : str
            The ID of the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agents.delete(
                organization_reference_id="organizationReferenceId",
                agent_reference_id="agentReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(
            organization_reference_id, agent_reference_id, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AgentUser,
    AgentUserField,
    AgentUserFilter,
    AgentUserSearchRequest,
    AgentUserSearchResponse,
    AppUserSummary,
)

__all__ = [
    "AgentUser",
    "AgentUserField",
    "AgentUserFilter",
    "AgentUserSearchRequest",
    "AgentUserSearchResponse",
    "AppUserSummary",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..commons.types.event_field import EventField
from ..commons.types.event_filter import EventFilter
from ..commons.types.event_request import EventRequest
from ..commons.types.event_response import EventResponse
from ..commons.types.events_search_response import EventsSearchResponse
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawEventsClient, RawEventsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEventsClient
        """
        return self._raw_client

    def create(
        self, *, request: EventRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> EventResponse:
        """
        Create a new event

        Parameters
        ----------
        request : EventRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventResponse
            A response indicating the event was successfully accepted

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import (
            EntityIdBase,
            EventRequest_UserEvent,
            EventUserInfoBase,
        )

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.events.create(
            request=EventRequest_UserEvent(
                id=EntityIdBase(
                    reference_id="x",
                ),
                event_name="BUTTON_CLICKED",
                user_info=EventUserInfoBase(
                    id=EntityIdBase(
                        reference_id="x",
                    ),
                ),
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def search(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EventsSearchResponse:
        """
        Search events

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventsSearchResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.events.search()
        """
        _response = self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    def get(
        self, event_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> EventResponse:
        """
        Retrieve details of a specific Event item by its ID.

        Parameters
        ----------
        event_id : str
            The ID of the Event to get.

        app_id : str
            The App ID of the Event to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventResponse
            Details of the specified Event

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.events.get(
            event_id="eventId",
            app_id="appId",
        )
        """
        _response = self._raw_client.get(event_id, app_id=app_id, request_options=request_options)
        return _response.data

    def export(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Export events to a CSV file.

        This will output a summary of each event that matches the supplied filter. A maximum of 10,000 events can be exported at a time. For most use cases it is recommended to use the search API instead and convert the JSON response to your desired format. The CSV format may change over time and should not be relied upon by code consumers.

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            A CSV containing one event per row

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.events.export()
        """
        with self._raw_client.export(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        ) as r:
            yield from r.data


class AsyncEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEventsClient
        """
        return self._raw_client

    async def create(
        self, *, request: EventRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> EventResponse:
        """
        Create a new event

        Parameters
        ----------
        request : EventRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventResponse
            A response indicating the event was successfully accepted

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import (
            EntityIdBase,
            EventRequest_UserEvent,
            EventUserInfoBase,
        )

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.events.create(
                request=EventRequest_UserEvent(
                    id=EntityIdBase(
                        reference_id="x",
                    ),
                    event_name="BUTTON_CLICKED",
                    user_info=EventUserInfoBase(
                        id=EntityIdBase(
                            reference_id="x",
                        ),
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def search(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EventsSearchResponse:
        """
        Search events

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventsSearchResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.events.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    async def get(
        self, event_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> EventResponse:
        """
        Retrieve details of a specific Event item by its ID.

        Parameters
        ----------
        event_id : str
            The ID of the Event to get.

        app_id : str
            The App ID of the Event to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventResponse
            Details of the specified Event

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.events.get(
                event_id="eventId",
                app_id="appId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(event_id, app_id=app_id, request_options=request_options)
        return _response.data

    async def export(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Export events to a CSV file.

        This will output a summary of each event that matches the supplied filter. A maximum of 10,000 events can be exported at a time. For most use cases it is recommended to use the search API instead and convert the JSON response to your desired format. The CSV format may change over time and should not be relied upon by code consumers.

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            A CSV containing one event per row

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.events.export()


        asyncio.run(main())
        """
        async with self._raw_client.export(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        ) as r:
            async for _chunk in r.data:
                yield _chunk



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .entity_id_filter import EntityIdFilter
from .user_data import UserData


class UserDataWithReference(UserData):
    user_id: typing_extensions.Annotated[EntityIdFilter, FieldMetadata(alias="userId")] = pydantic.Field()
    """
    Reverse index containing appId and referenceId data that identifies this app user
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .app_user import AppUser
from .entity_id import EntityId
from .user_data_with_reference import UserDataWithReference


class AppUserResponse(AppUser):
    """
    Examples
    --------
    from mavenagi.commons import (
        AppUserIdentifier,
        AppUserResponse,
        EntityId,
        EntityIdFilter,
        UserData,
        UserDataWithReference,
    )

    AppUserResponse(
        user_id=EntityId(
            reference_id="user-0",
            app_id="myapp",
            organization_id="acme",
            agent_id="support",
            type="USER_PROFILE",
        ),
        agent_user_id="aus_1234567890",
        identifiers=[
            AppUserIdentifier(
                value="joe@myapp.com",
                type="EMAIL",
            )
        ],
        data={
            "name": UserData(
                value="Joe",
                visibility="VISIBLE",
            )
        },
        all_user_data={"myapp": {"name": "Joe"}},
        default_user_data={"name": "Joe"},
        agent_user_data={
            "name": [
                UserDataWithReference(
                    value="Joe",
                    visibility="VISIBLE",
                    user_id=EntityIdFilter(
                        app_id="myapp",
                        reference_id="user0",
                    ),
                )
            ]
        },
    )
    """

    user_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="userId")] = pydantic.Field()
    """
    ID that uniquely identifies this user
    """

    agent_user_id: typing_extensions.Annotated[str, FieldMetadata(alias="agentUserId")] = pydantic.Field()
    """
    The ID of the agent user (merged view) that this app user is associated with
    """

    all_user_data: typing_extensions.Annotated[
        typing.Dict[str, typing.Dict[str, str]], FieldMetadata(alias="allUserData")
    ] = pydantic.Field()
    """
    Data from all apps
    """

    default_user_data: typing_extensions.Annotated[typing.Dict[str, str], FieldMetadata(alias="defaultUserData")] = (
        pydantic.Field()
    )
    """
    Default data for this user
    """

    agent_user_data: typing_extensions.Annotated[
        typing.Dict[str, typing.List[UserDataWithReference]], FieldMetadata(alias="agentUserData")
    ] = pydantic.Field()
    """
    All user data for this user, including reverse indexable user data
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class KnowledgeBaseRefreshRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.knowledge import KnowledgeBaseRefreshRequest

    KnowledgeBaseRefreshRequest(
        app_id="readme",
    )
    """

    app_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="appId")] = pydantic.Field(
        default=None
    )
    """
    The App ID of the knowledge base to refresh. If not provided the ID of the calling app will be used.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .attachment_response import AttachmentResponse
from .entity_id import EntityId
from .message_status import MessageStatus
from .user_message_base import UserMessageBase
from .user_message_response_state import UserMessageResponseState


class UserMessage(UserMessageBase):
    conversation_message_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationMessageId")] = (
        pydantic.Field()
    )
    """
    The ID that uniquely identifies this message within the conversation
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    The language of the message in ISO 639-1 code format
    """

    attachments: typing.List[AttachmentResponse] = pydantic.Field()
    """
    The attachments associated with the message
    """

    agent_user_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="agentUserId")] = (
        pydantic.Field(default=None)
    )
    """
    The ID of the agent user that created this message. More detail can be fetched via the agent user APIs. Will be empty only for legacy conversations.
    """

    user_display_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="userDisplayName")] = (
        pydantic.Field(default=None)
    )
    """
    The display name of the user who created this message. Only available for users who have saved name information.
    """

    status: MessageStatus = pydantic.Field()
    """
    The delivery status of the message. Only applicable to messages sent via the deliverMessage API.
    All other messages have an `UNKNOWN` status.
    
    * `SENT`: The message has been sent to the user.
    * `FAILED`: The message sending encountered an error.
    * `UNKNOWN`: The message status is unknown.
    """

    response_state: typing_extensions.Annotated[
        typing.Optional[UserMessageResponseState], FieldMetadata(alias="responseState")
    ] = pydantic.Field(default=None)
    """
    Only present on newer messaged where `userMessageType` is `USER`.
    Indicates the state of the answer to the user message.
    
    - `NOT_ASKED`: An answer was not requested for this user message.
    - `LLM_ENABLED`: An answer was requested for this user message and the LLM was enabled.
    - `LLM_DISABLED`: An answer was requested for this user message and the LLM was disabled.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
